@@ -1,5 +1,5 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
-   file Copyright.txt or https://cmake.org/licensing for details.  */
+   file LICENSE.rst or https://cmake.org/licensing for details.  */
 #include "CMakeSetupDialog.h"
 
 #include <cm/memory>
@@ -32,6 +32,7 @@
 
 #include "QCMake.h"
 #include "QCMakeCacheView.h"
+#include "QCMakeSizeType.h"
 
 #include "cmSystemTools.h"
 #include "cmVersion.h"
@@ -42,7 +43,7 @@
 #include "RegexExplorer.h"
 #include "WarningMessagesDialog.h"
 
-void OpenReferenceManual()
+void OpenReferenceManual(QString const& filename)
 {
   QString urlFormat("https://cmake.org/cmake/help/v%1.%2/");
   QUrl url(urlFormat.arg(QString::number(cmVersion::GetMajorVersion()),
@@ -50,15 +51,15 @@ void OpenReferenceManual()
 
   if (!cmSystemTools::GetHTMLDoc().empty()) {
     url = QUrl::fromLocalFile(
-      QDir(QString::fromLocal8Bit(cmSystemTools::GetHTMLDoc().data()))
-        .filePath("index.html"));
+      QDir(QString::fromStdString(cmSystemTools::GetHTMLDoc()))
+        .filePath(filename));
   }
 
   QDesktopServices::openUrl(url);
 }
 
 namespace {
-const QString PRESETS_DISABLED_TOOLTIP =
+QString const PRESETS_DISABLED_TOOLTIP =
   "This option is disabled because there are no available presets in "
   "CMakePresets.json or CMakeUserPresets.json.";
 }
@@ -212,7 +213,8 @@ CMakeSetupDialog::CMakeSetupDialog()
   QObject::connect(a, &QAction::triggered, this, &CMakeSetupDialog::doHelp);
   a->setShortcut(QKeySequence::HelpContents);
   a = HelpMenu->addAction(tr("CMake Reference Manual"));
-  QObject::connect(a, &QAction::triggered, this, OpenReferenceManual);
+  QObject::connect(a, &QAction::triggered, this,
+                   [] { OpenReferenceManual("index.html"); });
   a = HelpMenu->addAction(tr("About"));
   QObject::connect(a, &QAction::triggered, this, &CMakeSetupDialog::doAbout);
 
@@ -226,7 +228,8 @@ CMakeSetupDialog::CMakeSetupDialog()
   this->SourceDirectory->setCompleter(new QCMakeFileCompleter(this, true));
 
   // fixed pitch font in output window
-  QFont outputFont("Courier");
+  QFont outputFont("Courier New");
+  outputFont.setStyleHint(QFont::Monospace);
   this->Output->setFont(outputFont);
   this->ErrorFormat.setForeground(QBrush(Qt::red));
 
@@ -272,6 +275,8 @@ void CMakeSetupDialog::initialize()
   QObject::connect(this->BrowseSourceDirectoryButton,
                    &QAbstractButton::clicked, this,
                    &CMakeSetupDialog::doSourceBrowse);
+  QObject::connect(this->ReloadPresetsButton, &QAbstractButton::clicked,
+                   this->CMakeThread->cmakeInstance(), &QCMake::loadPresets);
   QObject::connect(this->BrowseBinaryDirectoryButton,
                    &QAbstractButton::clicked, this,
                    &CMakeSetupDialog::doBinaryBrowse);
@@ -347,8 +352,24 @@ void CMakeSetupDialog::initialize()
   if (!this->SourceDirectory->text().isEmpty() &&
       !this->DeferredPreset.isNull()) {
     this->onSourceDirectoryChanged(this->SourceDirectory->text());
+    if (!this->BinaryDirectory->lineEdit()->text().isEmpty()) {
+      this->onBinaryDirectoryChanged(
+        this->BinaryDirectory->lineEdit()->text());
+    }
   } else if (!this->SourceDirectory->text().isEmpty() ||
              !this->BinaryDirectory->lineEdit()->text().isEmpty()) {
+    if (this->SourceDirectory->text().isEmpty() &&
+        !cmSystemTools::FileIsFullPath(
+          this->BinaryDirectory->lineEdit()->text().toStdString())) {
+      // If the binary directory is relative, load the previous source path
+      // from the config
+      QSettings settings;
+      settings.beginGroup("Settings/StartPath");
+      QString srcDir = settings.value(QString("WhereSource")).toString();
+      this->SourceDirectory->blockSignals(true);
+      this->SourceDirectory->setText(srcDir);
+      this->SourceDirectory->blockSignals(false);
+    }
     this->onSourceDirectoryChanged(this->SourceDirectory->text());
     this->onBinaryDirectoryChanged(this->BinaryDirectory->lineEdit()->text());
   } else {
@@ -408,8 +429,13 @@ bool CMakeSetupDialog::prepareConfigure()
     }
   }
 
-  // remember path
-  this->addBinaryPath(dir.absolutePath());
+  // remember paths
+  this->addBinaryPath(
+    this->CMakeThread->cmakeInstance()->relativeBinaryDirectory());
+  QSettings settings;
+  settings.beginGroup("Settings/StartPath");
+  settings.setValue("WhereSource",
+                    this->CMakeThread->cmakeInstance()->sourceDirectory());
 
   return true;
 }
@@ -685,7 +711,7 @@ void CMakeSetupDialog::doSourceBrowse()
   }
 }
 
-void CMakeSetupDialog::updateSourceDirectory(const QString& dir)
+void CMakeSetupDialog::updateSourceDirectory(QString const& dir)
 {
   if (this->SourceDirectory->text() != dir) {
     this->SourceDirectory->blockSignals(true);
@@ -694,7 +720,7 @@ void CMakeSetupDialog::updateSourceDirectory(const QString& dir)
   }
 }
 
-void CMakeSetupDialog::updateBinaryDirectory(const QString& dir)
+void CMakeSetupDialog::updateBinaryDirectory(QString const& dir)
 {
   if (this->BinaryDirectory->currentText() != dir) {
     this->BinaryDirectory->blockSignals(true);
@@ -703,7 +729,7 @@ void CMakeSetupDialog::updateBinaryDirectory(const QString& dir)
   }
 }
 
-void CMakeSetupDialog::updatePresets(const QVector<QCMakePreset>& presets)
+void CMakeSetupDialog::updatePresets(QVector<QCMakePreset> const& presets)
 {
   if (this->Preset->presets() != presets) {
     this->Preset->blockSignals(true);
@@ -720,7 +746,7 @@ void CMakeSetupDialog::updatePresets(const QVector<QCMakePreset>& presets)
   }
 }
 
-void CMakeSetupDialog::updatePreset(const QString& name)
+void CMakeSetupDialog::updatePreset(QString const& name)
 {
   if (this->Preset->presetName() != name) {
     this->Preset->blockSignals(true);
@@ -729,26 +755,32 @@ void CMakeSetupDialog::updatePreset(const QString& name)
   }
 }
 
-void CMakeSetupDialog::showPresetLoadError(
-  const QString& dir, cmCMakePresetsGraph::ReadFileResult result)
+void CMakeSetupDialog::showPresetLoadError(QString const& dir,
+                                           QString const& message)
 {
   QMessageBox::warning(
     this, "Error Reading CMake Presets",
-    QString::fromLocal8Bit("Could not read presets from %1: %2")
-      .arg(dir, cmCMakePresetsGraph::ResultToString(result)));
+    QString("Could not read presets from %1: %2").arg(dir, message));
 }
 
 void CMakeSetupDialog::doBinaryBrowse()
 {
+  QString abs_path = this->BinaryDirectory->currentText();
+  if (!cmSystemTools::FileIsFullPath(abs_path.toStdString())) {
+    if (!this->SourceDirectory->text().endsWith("/")) {
+      abs_path = "/" + abs_path;
+    }
+    abs_path = this->SourceDirectory->text() + abs_path;
+  }
   QString dir = QFileDialog::getExistingDirectory(
-    this, tr("Enter Path to Build"), this->BinaryDirectory->currentText(),
+    this, tr("Enter Path to Build"), abs_path,
     QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
   if (!dir.isEmpty() && dir != this->BinaryDirectory->currentText()) {
     this->setBinaryDirectory(dir);
   }
 }
 
-void CMakeSetupDialog::setBinaryDirectory(const QString& dir)
+void CMakeSetupDialog::setBinaryDirectory(QString const& dir)
 {
   this->BinaryDirectory->setEditText(dir);
 }
@@ -758,7 +790,7 @@ void CMakeSetupDialog::setStartupBinaryDirectory(bool startup)
   this->StartupBinaryDirectory = startup;
 }
 
-void CMakeSetupDialog::onSourceDirectoryChanged(const QString& dir)
+void CMakeSetupDialog::onSourceDirectoryChanged(QString const& dir)
 {
   this->Output->clear();
   QMetaObject::invokeMethod(this->CMakeThread->cmakeInstance(),
@@ -766,7 +798,7 @@ void CMakeSetupDialog::onSourceDirectoryChanged(const QString& dir)
                             Q_ARG(QString, dir));
 }
 
-void CMakeSetupDialog::onBinaryDirectoryChanged(const QString& dir)
+void CMakeSetupDialog::onBinaryDirectoryChanged(QString const& dir)
 {
   QString title = QString(tr("CMake %1 - %2"));
   title = title.arg(cmVersion::GetCMakeVersion());
@@ -783,7 +815,7 @@ void CMakeSetupDialog::onBinaryDirectoryChanged(const QString& dir)
                             Q_ARG(QString, dir));
 }
 
-void CMakeSetupDialog::onBuildPresetChanged(const QString& name)
+void CMakeSetupDialog::onBuildPresetChanged(QString const& name)
 {
   QMetaObject::invokeMethod(this->CMakeThread->cmakeInstance(), "setPreset",
                             Qt::QueuedConnection, Q_ARG(QString, name),
@@ -791,17 +823,17 @@ void CMakeSetupDialog::onBuildPresetChanged(const QString& name)
   this->StartupBinaryDirectory = false;
 }
 
-void CMakeSetupDialog::setSourceDirectory(const QString& dir)
+void CMakeSetupDialog::setSourceDirectory(QString const& dir)
 {
   this->SourceDirectory->setText(dir);
 }
 
-void CMakeSetupDialog::setDeferredPreset(const QString& preset)
+void CMakeSetupDialog::setDeferredPreset(QString const& preset)
 {
   this->DeferredPreset = preset;
 }
 
-void CMakeSetupDialog::showProgress(const QString& /*msg*/, float percent)
+void CMakeSetupDialog::showProgress(QString const& /*msg*/, float percent)
 {
   percent = (percent * ProgressFactor) + ProgressOffset;
   this->ProgressBar->setValue(qRound(percent * 100));
@@ -813,7 +845,7 @@ void CMakeSetupDialog::showProgress(const QString& /*msg*/, float percent)
 #endif
 }
 
-void CMakeSetupDialog::error(const QString& msg)
+void CMakeSetupDialog::error(QString const& msg)
 {
   this->Output->setCurrentCharFormat(this->ErrorFormat);
   // QTextEdit will terminate the msg with a ParagraphSeparator, but it also
@@ -826,7 +858,7 @@ void CMakeSetupDialog::error(const QString& msg)
   this->Output->append(paragraph);
 }
 
-void CMakeSetupDialog::message(const QString& msg)
+void CMakeSetupDialog::message(QString const& msg)
 {
   this->Output->setCurrentCharFormat(this->MessageFormat);
   this->Output->append(msg);
@@ -873,7 +905,7 @@ bool CMakeSetupDialog::setupFirstConfigure()
     if (preset.setToolset) {
       dialog.setToolset(preset.toolset);
     }
-    dialog.setCompilerOption(CompilerOption::DefaultNative);
+    dialog.setCompilerOption(CompilerOption::DefaultPreset);
   }
 
   if (dialog.exec() == QDialog::Accepted) {
@@ -956,7 +988,7 @@ bool CMakeSetupDialog::setupFirstConfigure()
   return false;
 }
 
-void CMakeSetupDialog::updateGeneratorLabel(const QString& gen)
+void CMakeSetupDialog::updateGeneratorLabel(QString const& gen)
 {
   QString str = tr("Current Generator: ");
   if (gen.isEmpty()) {
@@ -994,8 +1026,7 @@ void CMakeSetupDialog::doAbout()
     "CMake suite maintained and supported by Kitware (kitware.com/cmake).\n"
     "Distributed under terms of the BSD 3-Clause License.\n"
     "\n"
-    "CMake GUI maintained by csimsoft,\n"
-    "built using Qt %2 (qt-project.org).\n"
+    "CMake GUI built using Qt %2 (qt-project.org).\n"
 #ifdef USE_LGPL
     "\n"
     "The Qt Toolkit is Copyright (C) The Qt Company Ltd.\n"
@@ -1031,7 +1062,7 @@ void CMakeSetupDialog::setExitAfterGenerate(bool b)
   this->ExitAfterGenerate = b;
 }
 
-void CMakeSetupDialog::addBinaryPath(const QString& path)
+void CMakeSetupDialog::addBinaryPath(QString const& path)
 {
   QString cleanpath = QDir::cleanPath(path);
 
@@ -1060,7 +1091,7 @@ void CMakeSetupDialog::dragEnterEvent(QDragEnterEvent* e)
     return;
   }
 
-  const QMimeData* dat = e->mimeData();
+  QMimeData const* dat = e->mimeData();
   QList<QUrl> urls = dat->urls();
   QString file = urls.count() ? urls[0].toLocalFile() : QString();
   if (!file.isEmpty() &&
@@ -1079,7 +1110,7 @@ void CMakeSetupDialog::dropEvent(QDropEvent* e)
     return;
   }
 
-  const QMimeData* dat = e->mimeData();
+  QMimeData const* dat = e->mimeData();
   QList<QUrl> urls = dat->urls();
   QString file = urls.count() ? urls[0].toLocalFile() : QString();
   if (file.endsWith("CMakeCache.txt", Qt::CaseInsensitive)) {
@@ -1113,17 +1144,17 @@ QStringList CMakeSetupDialog::loadBuildPaths()
   return buildPaths;
 }
 
-void CMakeSetupDialog::saveBuildPaths(const QStringList& paths)
+void CMakeSetupDialog::saveBuildPaths(QStringList const& paths)
 {
   QSettings settings;
   settings.beginGroup("Settings/StartPath");
 
-  int num = paths.count();
+  cm_qsizetype num = paths.count();
   if (num > 10) {
     num = 10;
   }
 
-  for (int i = 0; i < num; i++) {
+  for (cm_qsizetype i = 0; i < num; i++) {
     settings.setValue(QString("WhereBuild%1").arg(i), paths[i]);
   }
 }
@@ -1340,10 +1371,11 @@ void CMakeSetupDialog::showUserChanges()
   dialog.exec();
 }
 
-void CMakeSetupDialog::setSearchFilter(const QString& str)
+void CMakeSetupDialog::setSearchFilter(QString const& str)
 {
   this->CacheValues->selectionModel()->clear();
-  this->CacheValues->setSearchFilter(str);
+  bool const valid = this->CacheValues->setSearchFilter(str);
+  QtCMake::setSearchFilterColor(this->Search, valid);
 }
 
 void CMakeSetupDialog::doOutputContextMenu(QPoint pt)
